==> alsa-driver-0.5.0.tar.gz: alsa-driver-0.5.0/kernel/seq/seq_lock.c <==
/*
 *  ALSA sequencer main module
 *  Copyright (c) 1999 by Takashi Iwai <iwai@ww.uni-erlangen.de>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "../../include/driver.h"
#include "seq_lock.h"


void snd_seq_sleep_in_lock(wait_queue_head_t *p, spinlock_t *lock)
{
	wait_queue_t wait;

	current->state = TASK_INTERRUPTIBLE;
	/* 2.3.x: init_waitqueue_entry(&wait, current); */
	wait.task = current;

	add_wait_queue(p, &wait);

	spin_unlock(lock);
	schedule();
	spin_lock_irq(lock);

	remove_wait_queue(p, &wait);
}

long snd_seq_sleep_timeout_in_lock(wait_queue_head_t *p, spinlock_t *lock, long timeout)
{
	wait_queue_t wait;

	current->state = TASK_INTERRUPTIBLE;
	/* 2.3.x: init_waitqueue_entry(&wait, current); */
	wait.task = current;

	add_wait_queue(p, &wait);

	spin_unlock(lock);
	timeout = schedule_timeout(timeout);
	spin_lock_irq(lock);

	remove_wait_queue(p, &wait);

	return timeout;
}

/*
 * initialize lock
 */
void snd_use_lock_init(snd_use_lock_t *p)
{
	p->used = 0;
	p->lock = SPIN_LOCK_UNLOCKED;
	init_waitqueue_head(&p->sleep);
}

/*
 * increment with irq save
 */
void snd_use_lock_use(snd_use_lock_t *p)
{
	unsigned long flags;

	spin_lock_irqsave(&p->lock, flags);
	p->used++;
	spin_unlock_irqrestore(&p->lock, flags);
}

/*
 * increment without irq save
 */
void snd_use_lock_inc(snd_use_lock_t *p)
{
	spin_lock(&p->lock);
	p->used++;
	spin_unlock(&p->lock);
}

/*
 * decrement
 */
void snd_use_lock_free(snd_use_lock_t *p)
{
	unsigned long flags;

	spin_lock_irqsave(&p->lock, flags);
	p->used--;
	if (p->used <= 0 && waitqueue_active(&p->sleep))
		wake_up(&p->sleep);
	spin_unlock_irqrestore(&p->lock, flags);
}

/*
 * wait for free - spin-locked
 */
void snd_use_lock_wait(snd_use_lock_t *p, unsigned long *rflags)
{
	spin_lock_irqsave(&p->lock, *rflags);
	while (p->used)
		snd_seq_sleep_in_lock(&p->sleep, &p->lock);
}

/*
 * wait for free
 */
void snd_use_lock_terminate(snd_use_lock_t *p)
{
	unsigned long flags;
	snd_use_lock_wait(p, &flags);
	snd_use_lock_wait_unlock(p, flags);
}

==> alsa-driver-0.5_0.5.12a.orig.tar.gz: alsa-driver-0.5.12a/kernel/seq/seq_lock.c <==
/*
 *  ALSA sequencer main module
 *  Copyright (c) 1999 by Takashi Iwai <tiwai@suse.de>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "../../include/driver.h"
#include "seq_lock.h"


void snd_seq_sleep_in_lock(wait_queue_head_t *p, spinlock_t *lock)
{
	wait_queue_t wait;

	set_current_state(TASK_INTERRUPTIBLE);
	init_waitqueue_entry(&wait, current);

	add_wait_queue(p, &wait);

	spin_unlock(lock);
	schedule();
	spin_lock_irq(lock);

	remove_wait_queue(p, &wait);
}

long snd_seq_sleep_timeout_in_lock(wait_queue_head_t *p, spinlock_t *lock, long timeout)
{
	wait_queue_t wait;

	set_current_state(TASK_INTERRUPTIBLE);
	init_waitqueue_entry(&wait, current);

	add_wait_queue(p, &wait);

	spin_unlock(lock);
	timeout = schedule_timeout(timeout);
	spin_lock_irq(lock);

	remove_wait_queue(p, &wait);

	return timeout;
}

/*
 * initialize lock
 */
void snd_use_lock_init(snd_use_lock_t *p)
{
	p->used = 0;
	spin_lock_init(&p->lock);
	init_waitqueue_head(&p->sleep);
}

/*
 * increment with irq save
 */
void snd_use_lock_use(snd_use_lock_t *p)
{
	unsigned long flags;

	spin_lock_irqsave(&p->lock, flags);
	p->used++;
	spin_unlock_irqrestore(&p->lock, flags);
}

/*
 * increment without irq save
 */
void snd_use_lock_inc(snd_use_lock_t *p)
{
	spin_lock(&p->lock);
	p->used++;
	spin_unlock(&p->lock);
}

/*
 * decrement
 */
void snd_use_lock_free(snd_use_lock_t *p)
{
	unsigned long flags;

	spin_lock_irqsave(&p->lock, flags);
	p->used--;
	if (p->used <= 0 && waitqueue_active(&p->sleep))
		wake_up(&p->sleep);
	spin_unlock_irqrestore(&p->lock, flags);
}

/*
 * wait for free - spin-locked
 */
void snd_use_lock_wait(snd_use_lock_t *p, unsigned long *rflags)
{
	spin_lock_irqsave(&p->lock, *rflags);
	while (p->used)
		snd_seq_sleep_in_lock(&p->sleep, &p->lock);
}

/*
 * wait for free
 */
void snd_use_lock_terminate(snd_use_lock_t *p)
{
	unsigned long flags;
	snd_use_lock_wait(p, &flags);
	snd_use_lock_wait_unlock(p, flags);
}
